#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <stdio.h>  
#include <stdlib.h>   

using namespace std;

#define INTERPOLATION
//#define POISSON


// globale Variablen
#ifdef INTERPOLATION
const int polDegree = 2;
const int N = polDegree + 1;
const int M = 200;
#endif

#ifdef POISSON
const int N = 250;
const int M = N + 2;
const int BW = 3;
#endif

//=========================================================

// schreibe 2D Datensatz
void write2D( const double x[M], const double y[M], const char* filename ) {
  ofstream out ( filename );
  for( int i = 0; i < M; i++ )
    out << x[i] << " " << y[i] << endl;
}

// erzeuge eine Folge (x_i, y_i) mit y_i = sin(x_i) und i = 1, ..., M
void sampleSinusData( double x[M], double y[M] ){
  double eps = 1.;
  double min = 0 - eps;
  double max = 2. * M_PI + eps;
  double h = 1. * (max - min) / (M-1);
  
  for( int i = 0; i < M; i++ ){
    x[i] = min + i * h;
    y[i] = sin( x[i] );
  }
}

// erzeuge eine Folge (x_i, y_i) mit y_i = a_n (x_i)^n + ... + a_1 x_i + a_0, fuer i = 1, ..., M
void samplePolynomialData( double coeff[N], double x[M], double y[M] ) {
  double eps = 1.;
  double min = 0 - eps;
  double max = 2. * M_PI + eps;
  double h = 1. * (max - min) / (M-1);
  
  for( int i = 0; i < M; i++ ){
    x[i] = min + i * h;
    double potX = 1.;
    y[i] = 0;
    for( int j = 0; j < N; j++ ){
      y[i] += coeff[j] * potX;
      potX *= x[i];
    }
  }
}

//=========================================================
//=========================================================
//=========================================================

//TODO Programmieren Sie die LR-Zerlegung mit Pivotisierung,
// indem Sie die Matrizen L und R in der Matrix A speichern.
void computeLR( double A[N][N], int per[N] ) {

}

// TODO Fuehren Sie die Gauss Elimination aus indem Sie zunaechst die LR-Zerlegung 
// berechnen und dann Vorwaerts- und Rueckwaertseinsetzen.
// Hinweis: Die rechte Seite muss entsprechend der Pivotisierung permutiert werden!
void GaussElimination( double A[N][N], double b[N], double u[N] ){  
 
  // LR-Zerlegung
  int per[N];
  computeLR( A, per );   
  
  //TODO Vorwaerts- und Rueckwaertseinsetzen
}

//=========================================================

// Stellen Sie fuer festes N das LGS Ax = b mit NxN-Matrix A auf:
// a_{N-1} (x_i)^{N-1} + a_1 x_1 + a_0 = b_i, i = 0, ..., N-1
void buildSystemForSinusInterpolation( double A[N][N], double b[N] ) {

}

#ifdef POISSON
//TODO Programmieren Sie die LR-Zerlegung fuer eine Tridiagonalmatrix A.
// Die Matrix ist gegeben durch drei Vektoren, die Matrizen L und R sollen auch in A gespeichert werden.
// Sie duerfen annehmen, dass keine Pivotisierung noetig ist (gegeben fuer das Poisson-Problem!)
void computeLRForBandmatrixWithoutPivot( double A[BW][N] ) {

}

// TODO Fuehren Sie die Gauss Elimination aus indem Sie zunaechst die LR-Zerlegung 
// berechnen und dann Vorwaerts- und Rueckwaertseinsetzen.
void GaussEliminationForBandmatrixWithoutPivot( double A[BW][N], double b[N], double u[N] ){  
   
}

//TODO
void buildSystemForPoissonProblem( double A[N][N], double b[N] ) {

}

//TODO
void buildSystemForPoissonProblemBM( double A[BW][N], double b[N] ) {

}
#endif


//=========================================================
//=========================================================
//=========================================================

#ifdef INTERPOLATION 
// Hauptprogramm
int main () {
    
  static double A[N][N];
  static double b[N];
  static double u[N];    
    
  static double x[M];
  static double y[M];
  static double z[M];
  
  buildSystemForSinusInterpolation( A, b );  
  GaussElimination( A, b, u );  

  // output
  sampleSinusData( x, y );
  samplePolynomialData( u, x, z );
    
  write2D( x, y, "data1.dat" );
  write2D( x, z, "data2.dat" );

  return 0;
}
#endif

//=========================================================
//=========================================================
//=========================================================

#ifdef POISSON  
// Hauptprogramm
int main () {

  static double A[N][N];
  static double B[BW][N];
  static double b[N];
  static double u[N];    
    
  static double x[M];
  static double y[M];
  
  clock_t begin, end;
  
  // 
  buildSystemForPoissonProblem( A, b );  
  begin = clock();   
  GaussElimination( A, b, u );  
  end = clock();  
  cerr << "Time for greedy Gauss = " << double(end - begin) / CLOCKS_PER_SEC << "s." << endl;

  // casting
  for( int j = 0; j < M; j++ )
    x[j] = 1. * j / (M-1); 
  
  y[0] = y[N+1] = 0;
  for( int j = 0; j < N; j++ )
    y[j+1] = u[j];
    
  write2D( x, y, "data1.dat" );
  write2D( x, y, "data2.dat" );
  
  //  
  buildSystemForPoissonProblemBM( B, b );      
  begin = clock();
  GaussEliminationForBandmatrixWithoutPivot( B, b, u );
  end = clock();  
  cerr << "Time for Gauss using band structure = " << double(end - begin) / CLOCKS_PER_SEC << "s." << endl;

  return 0;
}
#endif

